(document:surround "/std/base")
(document:insert "/std/functions")

; Comments:
;  -- labels are not used in the list, use names + change name of common settings object

;;;

;; making rows for listbox
(define (make-row cmd)
  (let ((obj (woo-get-option cmd 'name)))
    (if (string=? obj "__common_settings__")
      (cons (_ "Common settings") "")
      (cons obj (if (woo-get-option cmd 'state) "on" "off")))))

(define (curr-obj)
  (let ((obj (main-listbox-id text)))
    (if (string=? obj (_ "Common settings"))
      "__common_settings__"
      obj)))

;; write down all settings via backend
(define (apply-to-backend url)
  (changed? value #f)
  (woo-write (string-append "/xinetd/" url)
      'state       (state-id value)
      'user        (user-id text)
      'group       (group-id text)
      'server      (server-id text)
      'server_args (server_args-id text)
      'rlimit_as   (rlimit_as-id text)
      'instances   (instances-id text)
      'per_source  (per_source-id text)
      'only_from   (only_from-id text) ))


;; reset all settings using data from backend
(define (reset-from-backend url)
  (changed? value #f)
  (let ((data (woo-read-first (string-append "/xinetd/" url))))

      (internal? value      (woo-get-option data 'type))

      (label-id text        (string-append "<b>" (main-listbox-id text)
                               (if (internal? value)
                                 (string-append " (" (_"internal service") ")")
                                 "") "</b>" ))
      (description-id text  (woo-get-option data 'description))
      (state-id value       (woo-get-option data 'state))

      (user-id text         (woo-get-option data 'user))
      (group-id text        (woo-get-option data 'group))
      (server-id text       (woo-get-option data 'server))
      (server_args-id text  (woo-get-option data 'server_args))
      (rlimit_as-id text    (woo-get-option data 'rlimit_as))
      (instances-id text    (woo-get-option data 'instances))
      (per_source-id text   (woo-get-option data 'per_source))
      (only_from-id text    (woo-get-option data 'only_from))
))

(define (reset-list)
  (let ((cur (main-listbox-id current)))
   (main-listbox-id
     rows (map make-row (woo-list "/xinetd"))
     current cur)))

(define (apply-and-reset backend)
  (apply-to-backend backend)
  (reset-from-backend backend)
  (reset-list))

;;

(define (make-cb label)
  (checkbox label (when toggled (changed? value #t))))

;; making widgets
(document:id internal? (checkbox value #f visibility #f name "internal?"))
(document:id common?   (checkbox value #t visibility #f name "common?"))
(document:id changed?  (checkbox value #f visibility #f))
(document:id old-obj   (edit     value "" visibility #f))


(splitbox columns "30;70"

  (document:id main-listbox-id (listbox
      header (vector (_ "Services"))
      current 0
      (when selected
        (if (changed? value)
          (if (eq? 'yes (document:popup-question (_"There are unsaved changes in the service settings. Do you want to apply these changes?") 'yes 'no))
            (apply-and-reset (old-obj value))
            (reset-from-backend (old-obj value))))

        (reset-from-backend (curr-obj))
        (common? value (string=? (curr-obj) "__common_settings__"))
        (update-effect)
        (old-obj value (curr-obj)))))

  (groupbox ""
    (document:id label-id (label ""))
    (document:id description-id (textbox "" alterability #f))

    (gridbox columns "0;100"

    (spacer)
    (document:id state-id
      (checkbox (_ "Enable service") name "state-id"
      ;(when toggled (changed? value #t))
      ))

    (label text (_ "User:") align "right" name "user-id")
    (document:id user-id
      (edit "" name "user-id" (when changed (changed? value #t))))

    (label text (_ "Group:") align "right" name "group-id")
    (document:id group-id
      (edit "" name "group-id" (when changed (changed? value #t))))

    (label text (_ "Server:") align "right" name "server-id")
    (document:id server-id
      (edit "" name "server-id" (when changed (changed? value #t))))

    (label text (_ "Server args:") align "right" name "server_args-id")
    (document:id server_args-id
      (edit "" name "server_args-id" (when changed (changed? value #t))))

    (label text (_ "Rlimit as:") align "right" name "rlimit_as-id")
    (document:id rlimit_as-id
      (edit "" name "rlimit_as-id" (when changed (changed? value #t))))

    (label text (_ "Instances:") align "right" name "instances-id")
    (document:id instances-id
      (edit "" name "instances-id" (when changed (changed? value #t))))

    (label text (_ "Per source:") align "right" name "per_source-id")
    (document:id per_source-id
      (edit "" name "per_source-id" (when changed (changed? value #t))))

    (label text (_ "Only from:") align "right" name "only_from-id")
    (document:id only_from-id
      (edit "" name "only_from-id" (when changed (changed? value #t))))

    (label(_ "This dialog changes only configuration file for xinetd. You must restart running xinetd to make it reread new configuration.") text-wrap #t colspan 2 )

    (spacer)

    (gridbox columns "0;0;100"
      (button (_ "Apply") (when clicked (apply-and-reset (curr-obj))))
      (button (_ "Reset") (when clicked (reset-from-backend (curr-obj))))
      (spacer)
      ))))

(document:root (when loaded
  (reset-list)
  (reset-from-backend (curr-obj))
  (old-obj value (curr-obj))
  (init-effect)
))

(effect-hide "state-id"       "common?" #t)
(effect-hide "user-id"        "common?" #t)
(effect-hide "group-id"       "common?" #t)
(effect-hide "server-id"      "common?" #t)
(effect-hide "server_args-id" "common?" #t)
(effect-hide "rlimit_as-id"   "common?" #t)

(effect-disable "user-id"        "state-id" #f)
(effect-disable "group-id"       "state-id" #f)
(effect-disable "server-id"      "state-id" #f)
(effect-disable "server_args-id" "state-id" #f)
(effect-disable "rlimit_as-id"   "state-id" #f)
(effect-disable "instances-id"   "state-id" #f)
(effect-disable "per_source-id"  "state-id" #f)
(effect-disable "only_from-id"   "state-id" #f)

(effect-hide "user-id"           "internal?" #t)
(effect-hide "group-id"          "internal?" #t)
(effect-hide "server-id"         "internal?" #t)
(effect-hide "server_args-id"    "internal?" #t)
(effect-show "internal_label-id" "internal?" #t)
